@@ -44,7 +44,6 @@ use std::{collections::hash_map::Entry, fmt, fs, io, str};
 use ahash::AHashMap;
 use byteorder::{BigEndian, WriteBytesExt};
 use serde::Deserialize;
-use simdutf8::basic::from_utf8;
 
 use crate::{
     config::{Config, Delimiter},
@@ -202,7 +201,10 @@ impl<R: io::Read + io::Seek> ValueIndex<R> {
             // indexes in one pass.
             row_idx.write_u64::<BigEndian>(row.position().unwrap().byte())?;
 
-            let fields: Vec<_> = sel.select(&row).map(|v| transform(v, casei)).collect();
+            let fields: Vec<_> = sel
+                .select(&row)
+                .map(|v| util::transform(v, casei))
+                .collect();
             if !fields.iter().any(std::vec::Vec::is_empty) {
                 match val_idx.entry(fields) {
                     Entry::Vacant(v) => {
@@ -248,20 +250,5 @@ impl<R> fmt::Debug for ValueIndex<R> {
 
 #[inline]
 fn get_row_key(sel: &Selection, row: &csv::ByteRecord, casei: bool) -> Vec<ByteString> {
-    sel.select(row).map(|v| transform(v, casei)).collect()
-}
-
-#[inline]
-fn transform(bs: &[u8], casei: bool) -> ByteString {
-    let s = from_utf8(bs).unwrap_or_default();
-    if casei {
-        let norm: String = s
-            .trim()
-            .chars()
-            .map(|c| c.to_lowercase().next().unwrap())
-            .collect();
-        norm.into_bytes()
-    } else {
-        s.trim().as_bytes().to_vec()
-    }
+    sel.select(row).map(|v| util::transform(v, casei)).collect()
 }
