@@ -77,7 +77,9 @@ use crate::{
     config::{Config, Delimiter, SeekRead},
     index::Indexed,
     select::{SelectColumns, Selection},
-    util, CliResult,
+    util,
+    util::ByteString,
+    CliResult,
 };
 
 #[derive(Deserialize)]
@@ -404,7 +406,10 @@ impl<R: io::Read + io::Seek> ValueIndex<R> {
             // indexes in one pass.
             row_idx.write_u64::<BigEndian>(row.position().unwrap().byte())?;
 
-            let fields: Vec<_> = sel.select(&row).map(|v| transform(v, casei)).collect();
+            let fields: Vec<_> = sel
+                .select(&row)
+                .map(|v| util::transform(v, casei))
+                .collect();
             if nulls || !fields.iter().any(std::vec::Vec::is_empty) {
                 match val_idx.entry(fields) {
                     Entry::Vacant(v) => {
@@ -450,25 +455,5 @@ impl<R> fmt::Debug for ValueIndex<R> {
 
 #[inline]
 fn get_row_key(sel: &Selection, row: &csv::ByteRecord, casei: bool) -> Vec<ByteString> {
-    sel.select(row).map(|v| transform(v, casei)).collect()
-}
-
-pub type ByteString = Vec<u8>;
-
-#[inline]
-pub fn transform(bs: &[u8], casei: bool) -> ByteString {
-    if let Ok(s) = str::from_utf8(bs) {
-        if casei {
-            let norm: String = s
-                .trim()
-                .chars()
-                .map(|c| c.to_lowercase().next().unwrap())
-                .collect();
-            norm.into_bytes()
-        } else {
-            s.trim().as_bytes().to_vec()
-        }
-    } else {
-        bs.to_vec()
-    }
+    sel.select(row).map(|v| util::transform(v, casei)).collect()
 }
