@@ -117,6 +117,8 @@ Common options:
                            in statistics.
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
+    --no-memcheck          Do not check if there is enough memory to load the
+                           entire CSV into memory.
 "#;
 
 /*
@@ -150,6 +152,7 @@ use itertools::Itertools;
 use once_cell::sync::OnceCell;
 use qsv_dateparser::parse_with_preference;
 use serde::Deserialize;
+use simdutf8::basic::from_utf8;
 use stats::{merge_all, Commute, MinMax, OnlineStats, Unsorted};
 use threadpool::ThreadPool;
 
@@ -182,6 +185,7 @@ pub struct Args {
     pub flag_output:          Option<String>,
     pub flag_no_headers:      bool,
     pub flag_delimiter:       Option<Delimiter>,
+    pub flag_no_memcheck:     bool,
 }
 
 static INFER_DATE_FLAGS: once_cell::sync::OnceCell<Vec<bool>> = OnceCell::new();
@@ -218,7 +222,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             || args.flag_quartiles
             || args.flag_mad
         {
-            util::mem_file_check(&path)?;
+            util::mem_file_check(&path, false, args.flag_no_memcheck)?;
         }
     }
 
@@ -339,20 +343,19 @@ impl Args {
     {
         let mut stats = self.new_stats(sel.len());
 
-        // amortize allocation
-        #[allow(unused_assignments)]
-        let mut record = csv::ByteRecord::with_capacity(1000, sel.len());
-        it.for_each(|row| {
-            record = unsafe { row.unwrap_unchecked() };
-            sel.select(&record).enumerate().for_each(|(i, field)| {
-                unsafe {
-                    // we use unchecked here so we skip unnecessary bounds checking
-                    stats
-                        .get_unchecked_mut(i)
-                        .add(field, *INFER_DATE_FLAGS.get_unchecked().get_unchecked(i));
-                }
-            });
-        });
+        // we use unsafe here so we skip unnecessary bounds checking
+        // that we are certain is safe
+        unsafe {
+            for row in it {
+                sel.select(&row.unwrap_unchecked())
+                    .enumerate()
+                    .for_each(|(i, field)| {
+                        stats
+                            .get_unchecked_mut(i)
+                            .add(field, *INFER_DATE_FLAGS.get_unchecked().get_unchecked(i));
+                    });
+            }
+        }
         stats
     }
 
@@ -477,7 +480,7 @@ fn init_date_inference(
 
             let mut infer_date_flags: Vec<bool> = Vec::with_capacity(headers.len());
             for header in headers {
-                let header_str = from_bytes::<String>(header).to_lowercase();
+                let header_str = from_bytes::<String>(header).unwrap().to_lowercase();
                 let mut date_found = false;
                 for whitelist_item in &whitelist {
                     if header_str.contains(whitelist_item) {
@@ -635,7 +638,7 @@ impl Stats {
                         };
                     }
                 } else {
-                    let n = from_bytes::<f64>(sample);
+                    let n = from_bytes::<f64>(sample).unwrap();
                     if let Some(v) = self.median.as_mut() {
                         v.add(n);
                     }
@@ -1029,8 +1032,7 @@ impl FieldType {
             return (FieldType::TString, None);
         }
 
-        // we skip utf8 validation since we say we only work with utf8
-        let string = unsafe { str::from_utf8_unchecked(sample) };
+        let string = from_utf8(sample).unwrap();
 
         if current_type == FieldType::TFloat
             || current_type == FieldType::TInteger
@@ -1137,7 +1139,7 @@ impl TypedSum {
         #[allow(clippy::cast_precision_loss)]
         match typ {
             TFloat => {
-                let float: f64 = from_bytes::<f64>(sample);
+                let float: f64 = from_bytes::<f64>(sample).unwrap();
                 match self.float {
                     None => {
                         self.float = Some((self.integer as f64) + float);
@@ -1149,10 +1151,12 @@ impl TypedSum {
             }
             TInteger => {
                 if let Some(ref mut float) = self.float {
-                    *float += from_bytes::<f64>(sample);
+                    *float += from_bytes::<f64>(sample).unwrap();
                 } else {
                     // so we don't panic on overflow/underflow, use saturating_add
-                    self.integer = self.integer.saturating_add(from_bytes::<i64>(sample));
+                    self.integer = self
+                        .integer
+                        .saturating_add(from_bytes::<i64>(sample).unwrap());
                 }
             }
             _ => {}
@@ -1214,37 +1218,35 @@ impl TypedMinMax {
             return;
         }
         self.strings.add(sample.to_vec());
-        // we can use unwrap_unchecked with confidence
-        // below since we know the data type domains of the sample
-        match typ {
-            TString | TNull => {}
-            TFloat => {
-                let n = unsafe {
-                    str::from_utf8_unchecked(sample)
+        // we can use unwrap_unchecked below since we know the data type of the sample
+        unsafe {
+            match typ {
+                TString | TNull => {}
+                TFloat => {
+                    let n = from_utf8(sample)
+                        .unwrap_unchecked()
                         .parse::<f64>()
-                        .unwrap_unchecked()
-                };
+                        .unwrap_unchecked();
 
-                self.floats.add(n);
-                self.integers.add(n as i64);
-            }
-            TInteger => {
-                let n = unsafe {
-                    str::from_utf8_unchecked(sample)
-                        .parse::<i64>()
+                    self.floats.add(n);
+                    self.integers.add(n as i64);
+                }
+                TInteger => {
+                    let n = from_utf8(sample)
                         .unwrap_unchecked()
-                };
-                self.integers.add(n);
-                #[allow(clippy::cast_precision_loss)]
-                self.floats.add(n as f64);
-            }
-            TDate | TDateTime => {
-                let n = unsafe {
-                    str::from_utf8_unchecked(sample)
                         .parse::<i64>()
+                        .unwrap_unchecked();
+                    self.integers.add(n);
+                    #[allow(clippy::cast_precision_loss)]
+                    self.floats.add(n as f64);
+                }
+                TDate | TDateTime => {
+                    let n = from_utf8(sample)
                         .unwrap_unchecked()
-                };
-                self.dates.add(n);
+                        .parse::<i64>()
+                        .unwrap_unchecked();
+                    self.dates.add(n);
+                }
             }
         }
     }
@@ -1330,7 +1332,10 @@ impl Commute for TypedMinMax {
 
 #[allow(clippy::inline_always)]
 #[inline(always)]
-fn from_bytes<T: FromStr>(bytes: &[u8]) -> T {
-    // we don't need to do UTF-8 validation as qsv requires UTF-8 encoding
-    unsafe { str::from_utf8_unchecked(bytes).parse().unwrap_unchecked() }
+fn from_bytes<T: FromStr>(bytes: &[u8]) -> Option<T> {
+    if let Ok(x) = from_utf8(bytes) {
+        x.parse().ok()
+    } else {
+        None
+    }
 }
