@@ -63,7 +63,6 @@ macro_rules! command_list {
     help        Show this usage message
     index       Create CSV index for faster access
     input       Read CSVs w/ special quoting, skipping, trimming & transcoding rules
-    luau*       Execute Luau script on CSV data
     pseudo      Pseudonymise the values of a column
     rename      Rename the columns of CSV data efficiently
     replace     Replace patterns in CSV data
@@ -80,7 +79,6 @@ macro_rules! command_list {
     validate    Validate CSV data for RFC4180-compliance or with JSON Schema
 
     NOTE: qsvdp ignores the --progressbar option for all commands.
-    luau is currently disabled until Datapusher+ can take advantage of it.
 
     sponsored by datHere - Data Infrastructure Engineering
 "
@@ -212,8 +210,8 @@ enum Command {
     Help,
     Index,
     Input,
-    #[cfg(feature = "luau")]
-    Luau,
+    // #[cfg(feature = "luau")]
+    // Luau,
     Pseudo,
     Rename,
     Replace,
@@ -258,8 +256,8 @@ impl Command {
             }
             Command::Index => cmd::index::run(argv),
             Command::Input => cmd::input::run(argv),
-            #[cfg(feature = "luau")]
-            Command::Luau => cmd::luau::run(argv),
+            // #[cfg(feature = "luau")]
+            // Command::Luau => cmd::luau::run(argv),
             Command::Pseudo => cmd::pseudo::run(argv),
             Command::Rename => cmd::rename::run(argv),
             Command::Replace => cmd::replace::run(argv),
